<?php 
	/*
	*	functions.php
	*/						 

  session_start();
  
	// Output debug strings?
	$debug = true;
	
	// register globals
	session_register("cmd");
	session_register("domain");

	/******************************************************************************************
	* show_debug($string)
	*/
	function show_debug($string) {
		global $debug;
		
		if ($debug) echo "<div style=\"background-color: #d3d3d3;>\">".$string."<br></div>\n";
	}

  function show_array($ar, $prefix="----", $recursive=0) {
		while (list($key,$value)=each($ar)) {
      if (gettype($value) == "array") {
        if ($recursive)
    			show_array ($value, $prefix." [$key] ", $recursive-1);
        else
    			show_debug ($prefix.$key."=>".$value);
      }
      else {
  			show_debug ($prefix.$key."=>".$value);
      }
    }
  }
  
	/******************************************************************************************
	* homedir returns path to homedir for given domain and uid 
	*/	
	function homedir($prefs, $domain, $uid) {									
		$path = $prefs["maildirs"]."/".$domain."/".$uid[0]."/".$uid;
		
		return $path;						
	}

	/******************************************************************************************
	* create_dn returns a string containing the desired dn
  *
  * $type may be one of
  * "uid" --> full dn for account is returned, needs additional paramameters $domain and $uid
  * "domain" --> dn for a maildomain is returned, needs additional paramameter $domain
  * "maildomains" --> dn for ou is returned which contains all of the maildomains
  * "maildrops" --> dn for ou with aliases is returned
  *
	*/
  function create_dn($prefs, $type, $domain="", $uid="") {

    switch ($type) {
      case "uid":
          return "uid=".$uid.","."ou=".$domain.",".$prefs["ldap"]["maildomains"].",".$prefs["ldap"]["basedn"];
        break;
        
      case "domain":
          return "ou=".$domain.",".$prefs["ldap"]["maildomains"].",".$prefs["ldap"]["basedn"];
        break;
        
      case "maildomains":
          return $prefs["ldap"]["maildomains"].",".$prefs["ldap"]["basedn"];
        break;
        
      case "maildrops":
          return $prefs["ldap"]["maildrops"].",".$prefs["ldap"]["basedn"];
        break;
    }
  }

	function cmp_accounts($entry_a, $entry_b){
		$sort_on = "cn";

//    show_debug("cmp_accounts(".$entry_a["cn"].",".$entry_b["cn"].")");
		return strcmp($entry_a[$sort_on], $entry_b[$sort_on]);
	}

	/******************************************************************************************
	* ldap_get_domains returns an array containing the domains
	*/
	function ldap_get_domains($prefs, $ldap) {
		if (! isset($ldap)) {
			$ldap = ldap_connect($prefs["ldap"]["host"], $prefs["ldap"]["port"]);
		}																																			 

		if ($ldap) {
			//bind anonymous
//			$bind = ldap_bind($ldap);

  		//create basedn for maildomains
  		$basedn = create_dn($prefs, "maildomains");
			
			//create filter for ldap_list; filter for objectclass=oU
			$filter = "(objectclass=organizationalUnit)"; 

			//search for organizationalUnits=Domains 
	    $sr = ldap_list($ldap, $basedn, $filter);	
			
  		if ($sr && ldap_count_entries($ldap, $sr)) {		 
  			// Show Domains
        $domains = ldap_get_entries($ldap, $sr);
  
        for ($i=0; $i<$domains["count"]; $i++) {
  				$result[$i] = $domains[$i]["ou"][0];
        }
    
        ldap_close($ldap);
  		}
		}
		else {
			$result = false;
		}

		return $result;	
	}
	
	/******************************************************************************************
	* ldap_entry2account returns an array containing mail,cn,userPassword,homeDirectory 
	*/	
	function ldap_entry2account($entry) {
		/*
		* We may have 2 "mail" attributes - one as mailadress for the mta and additionally one
		* for couriers authentication, in case of using old account names  
		*/

    //calc uid, which should be first entry of dn; if it doesnot exist, return full dn as uid
    if (ereg("uid=([^,]+),", $entry["dn"], $reg))
      $uid=$reg[1];
    else
      $uid=$entry["dn"];

		$result = array (
			"uid" => $uid,
			"cn" => utf8_decode($entry["cn"][0]),
			"userPassword" => $entry["userpassword"][0],
			"homeDirectory" => $entry["homedirectory"][0]
		);

    //split mailaccount and maildomain for each mailadress
    for ($i=0; $i<$entry["mail"]["count"]; $i++) {

      if (ereg("([^,]+)@([^,]+)", $entry["mail"][$i], $reg)) {
        $mail=$reg[1];
        $maildomain=$reg[2];
      }
      else {
        $mail=$entry["mail"][$i];
        $maildomain="";
      }
      
      $result["mail"][$i] = $mail;
      $result["maildomain"][$i] = $maildomain;
    }
    
		return ($result);
	}
	
	/******************************************************************************************
	* ldap_list_accounts returns array of accounts for given domain
	*/
	function ldap_list_accounts($prefs, $ldap, $domain) {
		if (! isset($ldap)) {
			$ldap = ldap_connect($prefs["ldap"]["host"], $prefs["ldap"]["port"]);
		}

		if ($ldap) {
			//bind anonymous
//			$bind = ldap_bind($ldap);

  		//create basedn for ldap_list as ou=<domain>,<basedn>
  		$basedn = create_dn($prefs, "domain", $domain);

			//create filter for ldap_list; filter for objectclass
			$filter = "(objectclass=CourierMailAccount)";

			//search for account, attribute mail is unique within domain
	    $sr = ldap_list($ldap, $basedn, $filter, array("cn","mail"));

			if ($sr && ldap_count_entries($ldap, $sr)) {
        $r = ldap_first_entry($ldap, $sr);
        while($r) {
  				$attrs = ldap_get_attributes($ldap, $r);

/*
          show_debug("add to result: cn=".$attrs["cn"][0].",mail[0]=".$attrs["mail"][0]);
*/
          unset($attrs["mail"]["count"]);
  				$result[ldap_get_dn($ldap, $r)] = array(
    					"cn" => utf8_decode($attrs["cn"][0]),
    					"mail" => $attrs["mail"]);

          $r =  ldap_next_entry($ldap, $r);
        }
        
        uasort($result, cmp_accounts);
			}
			else {
				$result = false;
			}
		}
		else {
			$result = false;
		}

		return $result;
  }
  
	/******************************************************************************************
	* ldap_get_account returns an account array for $uid@$domain
	*/	
	function ldap_get_account($prefs, $ldap, $domain, $uid) {
		if (! isset($ldap)) {
			$ldap = ldap_connect($prefs["ldap"]["host"], $prefs["ldap"]["port"]);
		}																																			 

		if ($ldap) {
			//bind anonymous
//			$bind = ldap_bind($ldap);

  		//create basedn for ldapsearch as ou=<domain>,<basedn>
  		$basedn = create_dn($prefs, "uid", $domain, $uid);

			//create filter for ldapsearch; filter for objectclass
			$filter = "(objectclass=CourierMailAccount)";

			//search for account, attribute mail is unique within domain 
	    $sr = ldap_search($ldap, $basedn, $filter);

			if ($sr && ldap_count_entries($ldap, $sr)) {
				$entries = ldap_get_entries($ldap, $sr);
				$result = ldap_entry2account($entries[0]);
			}
			else {
				$result = false;
			}  
		}
		else {
			$result = false;
		}

		return $result;	
	}
	
	/******************************************************************************************
	* ldap_set_account sets account 
	*/	
	function ldap_set_account($prefs, $ldap, $domain, $account) {
    show_debug("ldap_set_account");
    show_array($account, "account[] ", 1);
    
    //check for new account (uid is empty)
    if (strlen($account["uid"]) == 0) {
      //create uid
      show_debug("creating uid for new account");

      //use mailaccount for first entry with maildomain=domain
      for ($i=0; $i<count($account["mail"]); $i++) {
        if ($account["maildomain"][$i] == $domain) {
          $account["uid"] = $account["mail"][$i];
          break;
        }
      }
    }
    
    //connect to ldap
		if (! isset($ldap)) {
			show_debug ("connecting to ldap... host=".$prefs["ldap"]["host"].":".$prefs["ldap"]["port"]);
			$ldap = ldap_connect($prefs["ldap"]["host"], $prefs["ldap"]["port"]);
		}																																			 

		if ($ldap) {
			//bind  
			show_debug ("binding...");
			$bind = ldap_bind($ldap, $prefs["ldap"]["binddn"], $prefs["ldap"]["bindpw"]); 			

      //crypt password
			if (strpos($account["userPassword"], "{CRYPT}") === false) {
        show_debug("crypting password: ".$account["userPassword"]);
				$account["userPassword"] = "{CRYPT}".crypt($account["userPassword"]);
      }
				
  		//create dn as ou=<domain>,<basedn>
  		$dn = create_dn($prefs, "uid", $domain, $account["uid"]);

  		//set attributes
			if (strlen($account["homeDirectory"]) == 0)
				$account["homeDirectory"] = homedir($prefs, $domain, $account["uid"]);
						
			$entry = array(
				"objectclass" => "CourierMailAccount",
				"cn" => utf8_encode($account["cn"]),
				"homeDirectory" => $account["homeDirectory"],
				"userPassword" => $account["userPassword"],
				"uidNumber" => 0,
				"gidNumber" => 0
			);

      //create attributes for mailadresses
      for ($i=0, $j=0; $i<count($account["mail"]); $i++) {
        $mail = $account["mail"][$i];

        if (strlen($mail)>0) {
          if (strlen($account["maildomain"][$i]) > 0)
            $mail .= "@".$account["maildomain"][$i];

          $entry["mail"][$j++] = $mail;
          show_debug("added mail $mail");
        }
      }

      sort($entry["mail"]);

			//look, if account already exists
  		//create basedn for ldap_list as ou=<domain>,<basedn>
      $basedn = create_dn($prefs, "domain", $domain);

			//create filter for ldapsearch; filter for objectclass and mail
      //create filter to search for one of the given mailadresses
			$filter = "(&(objectclass=CourierMailAccount)(|";
      for ($i=0; $i<count($entry["mail"]); $i++) {
        $filter .= "(mail=".$entry["mail"][$i].")";
      }
      $filter .= "))";

			show_debug("Searching in ".$basedn);
			show_debug("Searching for ".$filter);

			//search for account, attribute mail should be unique within domain 
	    $sr = ldap_search($ldap, $basedn, $filter, array("mail"));

			if ($sr && ldap_count_entries($ldap, $sr)) {
				$entries = ldap_get_entries($ldap, $sr);
				
				//remove entries with dn other than calculated or modify...
				$modified = false;
				
				for ($i=0; $i<$entries["count"]; $i++) {
					if (strcmp($dn, $entries[$i]["dn"]) == 0) {
						show_debug("modify entry $dn");
						$result = ldap_modify($ldap, $dn, $entry);
						$modified = true;
					}
					else{
						show_debug("deleting ".$entries[$i]["dn"]);
						ldap_delete($ldap, $entries[$i]["dn"]);
					}																		 
				}				

				//... and eventually add new entry
				if (!$modified) {
					show_debug("add (1) entry $dn");
					$result = ldap_add($ldap, $dn, $entry);
				}
			}

			else {
				show_debug("add (2) entry $dn");
				$result = ldap_add($ldap, $dn, $entry);
			}
		}

		else {
			$result = false;
		}

		if ($result) {
			show_debug("exec (".$prefs["cmd_chk4maildir"]." ".$account["homeDirectory"].")");
			exec ($prefs["cmd_chk4maildir"]." ".$account["homeDirectory"], $execoutput);
																											
			while (list($key, $value) = each($execoutput)) {
				show_debug($key ."=>".$value);
			}
		} 
			
		if (!$result) 
			echo ldap_error($ldap)."<br>\n";
			
		return $result;	
	}
?>

